package com.example.duplicate.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.example.duplicate.controller.param.ArchiveOtherExtDto;
import com.example.duplicate.controller.param.PrintParam;
import com.example.duplicate.controller.vo.ArchiveMasterVo;
import com.example.duplicate.controller.vo.ArchiveOtherExtVo;
import com.example.duplicate.infrastructure.dao.ArchiveMasterMapper;
import com.example.duplicate.infrastructure.dao.ArchiveOtherExtMapper;
import com.example.duplicate.service.MedicalAdviceService;
import com.example.utils.CommonResult;
import com.example.utils.RedisMq;
import com.example.utils.SnowflakeIdWorker;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * @ClassName MedicalAdviceServiceImpl
 * @Description 医嘱任务实现类
 * @Author linjj
 * @Date 2024/8/2 15:48
 * @Version 1.0
 */
@Service
@Slf4j
public class MedicalAdviceServiceImpl implements MedicalAdviceService {

    @Value("${collectIdList}")
    private String collectIdList;
    @Autowired
    private ArchiveMasterMapper ArchiveMasterMapper;
    @Autowired
    private ArchiveOtherExtMapper archiveOtherExtMapper;
    @Autowired(required = false)
    private RedisMq redisMq;

    private String redisKey = "docus:task:topic_collect_%s_queue:%s";

    @Override
    public CommonResult<?> pollingAddTask() {
        //获取当前30分钟前出院病历生成医嘱任务
        List<ArchiveMasterVo> medicalAdviceTask = ArchiveMasterMapper.getMedicalAdviceTask();
        if (CollectionUtils.isEmpty(medicalAdviceTask)) {
            log.info("当前没有需要生成任务的病历");
            return CommonResult.failed("当前没有需要生成任务的病历");
        }
        PrintParam printParam = new PrintParam();
        //任务表入参
        ArchiveOtherExtDto dto = new ArchiveOtherExtDto();
        //任务生成时间
        Date date = new Date();
        SimpleDateFormat SycTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        //补偿所有采集器id集合
        List<String> collectIds = Arrays.asList(collectIdList.split(","));
        //生成任务
        for (ArchiveMasterVo tasks : medicalAdviceTask) {
            //增加所有采集id的任务
            for (String collectId : collectIds) {
                try {
                    //查询任务表是否存在，不存在新增
                    List<ArchiveOtherExtVo> others = archiveOtherExtMapper.getOtherByMidAndFlag(tasks.getId(), Integer.parseInt(collectId));
                    if (CollectionUtils.isEmpty(others)) {
                        //任务表新增
                        dto.setId(SnowflakeIdWorker.idWorker.nextId());
                        dto.setSycTime(SycTime.format(date));
                        dto.setOtherID("0");
                        dto.setSysFlag(Integer.parseInt(collectId));
                        dto.setSysUpdateTime(SycTime.format(date));
                        dto.setJzh(tasks.getPatientId());
                        dto.setZyh(tasks.getInpNo());
                        dto.setStatusFlag(0);
                        dto.setMID(tasks.getId());
                        if (collectId.equals("8")){
                            dto.setC1("长临医嘱");
                        }
                        if (collectId.equals("1")){
                            dto.setC1("护理记录");
                        }
                        //任务表新增任务
                        if (archiveOtherExtMapper.addOther(dto)) {
                            //存放队列消息
                            GenerateQueue(tasks, printParam, 5, collectId);
                            log.info("轮询增加任务成功,id为:" + tasks.getId());
                        }
                    }
                } catch (Exception e) {
                    log.error("轮询增加任务失败,id为:" + tasks.getId(), e);
                }
            }
        }
        return CommonResult.success("轮询完成已经增加任务");
    }


    //生成队列消息
    private void GenerateQueue(ArchiveMasterVo task, PrintParam data, Integer ispriority, String collectid) {
        data.setId(task.getId());
        data.setInpNo(task.getInpNo());
        data.setVisitId(task.getVisitId());
        data.setDeptName(task.getDeptName());
        data.setDischargeDateTime(task.getDischargeDateTime());
        data.setName(task.getName());
        data.setAdmissionDateTime(task.getAdmissionDateTime());
        String key = String.format(redisKey, collectid, ispriority);
        redisMq.push(key, JSON.toJSONString(data, SerializerFeature.WriteDateUseDateFormat, SerializerFeature.WriteMapNullValue));
    }
}
